@@ -360,12 +360,15 @@ monitor_daemon(pid_t daemon_pid)
                                (unsigned long int) daemon_pid, status_msg);
 
         if (child_ready) {
+            char *log_file = vlog_get_log_file();
+            vlog_close_log_file();
             int error;
             do {
                 retval = waitpid(daemon_pid, &status, 0);
                 error = retval == -1 ? errno : 0;
             } while (error == EINTR);
-            vlog_reopen_log_file();
+            vlog_set_log_file(log_file);
+            free(log_file);
             if (error) {
                 VLOG_FATAL("waitpid failed (%s)", ovs_strerror(error));
             }
@@ -977,7 +980,7 @@ daemon_set_new_user(const char *user_spec)
         }
 
         if (e != 0) {
-            VLOG_FATAL("%s: Failed to retrive user %s's uid (%s), aborting.",
+            VLOG_FATAL("%s: Failed to retrieve user %s's uid (%s), aborting.",
                        pidfile, user, ovs_strerror(e));
         }
         if (res == NULL) {
@@ -992,7 +995,7 @@ daemon_set_new_user(const char *user_spec)
         }
 
         if (e != 0) {
-            VLOG_FATAL("%s: Failed to retrive current user's name "
+            VLOG_FATAL("%s: Failed to retrieve current user's name "
                        "(%s), aborting.", pidfile, ovs_strerror(e));
         }
         user = xstrdup(pwd.pw_name);
