@@ -154,6 +154,7 @@ netdev_initialize(void)
         netdev_register_flow_api_provider(&netdev_offload_tc);
 #ifdef HAVE_AF_XDP
         netdev_register_provider(&netdev_afxdp_class);
+        netdev_register_provider(&netdev_afxdp_nonpmd_class);
 #endif
 #endif
 #if defined(__FreeBSD__) || defined(__NetBSD__)
@@ -386,25 +387,30 @@ netdev_open(const char *name, const char *type, struct netdev **netdevp)
     ovs_mutex_lock(&netdev_mutex);
     netdev = shash_find_data(&netdev_shash, name);
 
-    if (netdev &&
-        type && type[0] && strcmp(type, netdev->netdev_class->type)) {
+    if (netdev && type && type[0]) {
+        if (strcmp(type, netdev->netdev_class->type)) {
 
-        if (netdev->auto_classified) {
-            /* If this device was first created without a classification type,
-             * for example due to routing or tunneling code, and they keep a
-             * reference, a "classified" call to open will fail. In this case
-             * we remove the classless device, and re-add it below. We remove
-             * the netdev from the shash, and change the sequence, so owners of
-             * the old classless device can release/cleanup. */
-            if (netdev->node) {
-                shash_delete(&netdev_shash, netdev->node);
-                netdev->node = NULL;
-                netdev_change_seq_changed(netdev);
+            if (netdev->auto_classified) {
+                /* If this device was first created without a classification
+                 * type, for example due to routing or tunneling code, and they
+                 * keep a reference, a "classified" call to open will fail.
+                 * In this case we remove the classless device, and re-add it
+                 * below. We remove the netdev from the shash, and change the
+                 * sequence, so owners of the old classless device can
+                 * release/cleanup. */
+                if (netdev->node) {
+                    shash_delete(&netdev_shash, netdev->node);
+                    netdev->node = NULL;
+                    netdev_change_seq_changed(netdev);
+                }
+
+                netdev = NULL;
+            } else {
+                error = EEXIST;
             }
-
-            netdev = NULL;
-        } else {
-            error = EEXIST;
+        } else if (netdev->auto_classified) {
+            /* If netdev reopened with type "system", clear auto_classified. */
+            netdev->auto_classified = false;
         }
     }
 
@@ -1618,17 +1624,20 @@ netdev_get_custom_stats(const struct netdev *netdev,
     return error;
 }
 
-
-/* Attempts to set input rate limiting (policing) policy, such that up to
- * 'kbits_rate' kbps of traffic is accepted, with a maximum accumulative burst
- * size of 'kbits' kb. */
+/* Attempts to set input rate limiting (policing) policy, such that:
+ * - up to 'kbits_rate' kbps of traffic is accepted, with a maximum
+ *   accumulative burst size of 'kbits' kb; and
+ * - up to 'kpkts' kpps of traffic is accepted, with a maximum
+ *   accumulative burst size of 'kpkts' kilo packets.
+ */
 int
 netdev_set_policing(struct netdev *netdev, uint32_t kbits_rate,
-                    uint32_t kbits_burst)
+                    uint32_t kbits_burst, uint32_t kpkts_rate,
+                    uint32_t kpkts_burst)
 {
     return (netdev->netdev_class->set_policing
             ? netdev->netdev_class->set_policing(netdev,
-                    kbits_rate, kbits_burst)
+                    kbits_rate, kbits_burst, kpkts_rate, kpkts_burst)
             : EOPNOTSUPP);
 }
 
@@ -1983,6 +1992,22 @@ netdev_get_class(const struct netdev *netdev)
     return netdev->netdev_class;
 }
 
+/* Set the type of 'dpif' this 'netdev' belongs to. */
+void
+netdev_set_dpif_type(struct netdev *netdev, const char *type)
+{
+    netdev->dpif_type = type;
+}
+
+/* Returns the type of 'dpif' this 'netdev' belongs to.
+ *
+ * The caller must not free the returned value. */
+const char *
+netdev_get_dpif_type(const struct netdev *netdev)
+{
+    return netdev->dpif_type;
+}
+
 /* Returns the netdev with 'name' or NULL if there is none.
  *
  * The caller must free the returned netdev with netdev_close(). */
